import json
import re
from datetime import datetime

import requests
from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import models, transaction
from django.utils.module_loading import import_string
from django.db.models.signals import post_save
from django.dispatch import receiver

from .user_list import NotifyUserList
from .category import NotifyCategory
from .choices import TYPE, STATE
from .file import NotifyFile
from .template import NotifyTemplate
from ..exceptions import TemplatesNotExists, IsInstanceException
from ..mixins import UserNotifyMixin
from ..mixins.notify_method_mixin import NotifyMethodsMixin
from ..utils.send_data import SendDataService
from ..clients import SMSClient, EmailClient, CallClient, TelegramClient, ViberClient, PushClient, WhatsAppClient

NotifyMixin = import_string(getattr(settings, 'GARPIX_NOTIFY_MIXIN', 'garpix_notify.mixins.notify_mixin.NotifyMixin'))

User = get_user_model()


class Notify(NotifyMixin, UserNotifyMixin, NotifyMethodsMixin):
    """
    Уведомление
    """
    subject = models.CharField(max_length=255, default='', blank=True, verbose_name='Тема')
    text = models.TextField(verbose_name='Текст')
    html = models.TextField(verbose_name='HTML', blank=True, default='')

    user: User = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True, on_delete=models.SET_NULL,
                                   related_name='notifies', verbose_name='Пользователь (получатель)')
    email = models.EmailField(max_length=255, blank=True, null=True, verbose_name='Email Получателя',
                              help_text='Используется только в случае отсутствия указанного пользователя')
    sender_email = models.EmailField(max_length=255, blank=True, null=True, verbose_name='Email Отправителя')

    type = models.IntegerField(choices=TYPE.CHOICES, verbose_name='Тип')
    state = models.IntegerField(choices=STATE.CHOICES, default=STATE.WAIT, verbose_name='Состояние')
    category = models.ForeignKey(NotifyCategory, on_delete=models.CASCADE, related_name='notifies',
                                 verbose_name='Категория')
    event = models.IntegerField(choices=settings.CHOICES_NOTIFY_EVENT, blank=True, null=True, verbose_name='Событие')
    files = models.ManyToManyField(NotifyFile, verbose_name='Файлы')

    is_read = models.BooleanField(default=False, verbose_name='Прочитано')
    data_json = models.TextField(blank=True, null=True, verbose_name='Данные пуш-уведомления (JSON)')
    room_name = models.CharField(max_length=255, null=True, blank=True, verbose_name='Название комнаты')

    users_list = models.ManyToManyField(NotifyUserList, blank=True, verbose_name='Списки пользователей для рассылки')

    created_at = models.DateTimeField(auto_now_add=True, verbose_name='Дата создания')
    send_at = models.DateTimeField(blank=True, null=True, verbose_name='Время начала отправки')
    sent_at = models.DateTimeField(blank=True, null=True, verbose_name='Дата отправки')

    def __str__(self):
        return self.subject if self.subject and self.subject != '' else f'Уведомление № {self.id}'

    def _get_sender(self):
        if self.user:
            self.email = self.user.email if self.user.email else self.email
            self.phone = str(self.user.phone) if self.user.phone else str(self.phone)
            self.telegram_chat_id = self.user.telegram_chat_id if self.user.telegram_chat_id else self.telegram_chat_id
            self.viber_chat_id = self.user.viber_chat_id if self.user.viber_chat_id else self.viber_chat_id

        if self.phone is not None:
            self.phone = re.sub("[^0-9]", "", self.phone)

    def _start_send(self):  # noqa

        # Если передан пользователь, то перезаписываем данные (если они есть у пользователя)
        self._get_sender()

        if self.type == TYPE.EMAIL:
            EmailClient.send_email(self)
        elif self.type == TYPE.SMS:
            SMSClient.send_sms(self)
        elif self.type == TYPE.PUSH:
            PushClient.send_push(self)
        elif self.type == TYPE.TELEGRAM:
            TelegramClient.send_telegram(self)
        elif self.type == TYPE.VIBER:
            ViberClient.send_viber(self)
        elif self.type == TYPE.CALL:
            CallClient.send_call(self)
        elif self.type == TYPE.WHATSAPP:
            WhatsAppClient.send_whatsapp(self)

        self.save()

    @staticmethod
    def send(event: int, context: dict, user: User = None, email: str = None, phone: str = None,  # noqa: C901
             files: list = None, data_json: dict = None, viber_chat_id: str = None, room_name: str = None,
             notify_templates: list = None, send_at: datetime = None, send_now: bool = False, **kwargs) -> list:

        if user and not isinstance(user, User):
            raise IsInstanceException

        instance_list: list = []
        user_want_message_check = None

        # Сначала забираем те данные, которые передали с методом
        notify_user = user if user else None
        notify_email = email if email else None
        notify_phone = phone if phone else None
        notify_viber_chat_id = viber_chat_id if viber_chat_id else None

        local_context = context.copy()

        if hasattr(settings, 'NOTIFY_USER_WANT_MESSAGE_CHECK') and settings.NOTIFY_USER_WANT_MESSAGE_CHECK is not None:
            user_want_message_check = import_string(settings.NOTIFY_USER_WANT_MESSAGE_CHECK)

        data_json = json.dumps(data_json) if data_json is not None else None

        if notify_templates:
            templates = (
                NotifyTemplate.objects
                .select_related('category', 'user')
                .prefetch_related('user_lists')
                .filter(id__in=notify_templates, event=event, is_active=True)
            )
        else:
            templates = (
                NotifyTemplate.objects
                .select_related('category', 'user')
                .prefetch_related('user_lists')
                .filter(event=event, is_active=True)
            )

        if not templates.exists():
            raise TemplatesNotExists

        if files is None:
            files = []

        file_instances = []
        for f in files:
            instance = NotifyFile.objects.create(file=f)
            file_instances.append(instance)

        for template in templates:

            notify_users_lists = template.user_lists.all()

            # Формируем список основных получателей письма
            # Из шаблона, если методом не было передано данных
            template_user = template.user if template.user else None
            template_email = template.email if template.email else None
            template_phone = template.phone if template.phone else None
            template_viber_chat_id = template.viber_chat_id if template.viber_chat_id else None

            # Данные, которые указал пользователь у себя в профиле в приоритете
            if notify_user:
                notify_email = notify_user.email if notify_user.email else notify_email
                notify_phone = notify_user.phone if notify_user.phone else notify_phone
                notify_viber_chat_id = (
                    notify_user.viber_chat_id if notify_user.viber_chat_id else notify_viber_chat_id
                )

            if notify_user is None and template_user:
                notify_user = template.user if template.user else None
                notify_email = template.user.email if template.user.email else notify_email
                notify_phone = template.user.phone if template.user.phone else notify_phone
                notify_viber_chat_id = (
                    template.user.viber_chat_id if template.user.viber_chat_id else notify_viber_chat_id
                )

            if notify_email is None:
                notify_email = template_email

            if notify_phone is None:
                notify_phone = template_phone

            if notify_viber_chat_id is None:
                notify_viber_chat_id = template_viber_chat_id

            # Проверка, хочет ли пользователь получить сообщение
            if user_want_message_check is not None:  # noqa
                if not notify_users_lists.exists():
                    user_check = user_want_message_check(event, template.type, notify_user)
                    if not user_check:
                        continue
                else:
                    # Если у нас шаблон со списками, то передаем в функцию и тут формируем новый из тех пользователей,
                    # которым сообщения нужны
                    # Если список пустой, то отменяем отправку
                    notify_users_lists = user_want_message_check(
                        event, template.type, notify_user, notify_users_lists)
                    if not notify_users_lists:
                        continue
            # Добавляем пользователя в контекст, если его там не передали
            if notify_user is not None:
                if local_context is not None:
                    local_context.update({
                        'user': notify_user,
                    })
                else:
                    local_context = {
                        'user': notify_user,
                    }
            local_context['event_id'] = event

            # Проверка на наличие списка в шаблоне
            # Если в шаблоне передаются списки пользователей, то отдаем их уведомлениям
            # Если уведомление для одного пользователя, то создаем для одного
            # Также идет проверка на время оправки
            if send_at is not None:
                notify_send = send_at
            else:
                notify_send = template.send_at
            instance = Notify.objects.create(
                subject=template.render_subject(local_context),
                text=template.render_text(local_context),
                html=template.render_html(local_context),
                user=notify_user,
                email=notify_email,
                phone=notify_phone if notify_phone is not None else "",
                viber_chat_id=notify_viber_chat_id if notify_viber_chat_id is not None else "",
                type=template.type,
                event=template.event,
                category=template.category if template.category else None,
                data_json=data_json,
                send_at=notify_send,
                room_name=room_name,
                **kwargs
            )
            if notify_users_lists.exists():
                instance.users_list.add(*notify_users_lists)
            file_instance = instance.files
            for f in file_instances:
                file_instance.add(f)
            instance.save()

            if send_now:
                transaction.on_commit(lambda: instance._start_send())  # noqa

            instance_list.append(instance)
        return instance_list

    @staticmethod
    def call(phone: str, user: User = None, url: str = None, **kwargs):
        call_url_type: int = CallClient.get_url_type()

        send_data_service = SendDataService()

        if user and not isinstance(user, User):
            raise IsInstanceException

        if user is not None:
            phone = user.phone if user.phone else phone

        if url is not None:
            url = url
        else:
            url = send_data_service.url_dict_call[call_url_type]

        main_url = url.format(**send_data_service.operator_call[call_url_type], to=phone, **kwargs)
        response_url = requests.get(main_url)
        response_dict = response_url.json()
        value = CallClient.get_value_checker(response_dict)

        response = send_data_service.response_check(response=response_dict, operator_type=call_url_type, status=value)
        if value == "OK":
            return '{Code}'.format(**response)
        else:
            return None

    class Meta:
        verbose_name = 'Уведомление'
        verbose_name_plural = 'Уведомления'


@receiver(post_save, sender=Notify)
def system_post_save(sender, instance, created, **kwargs):
    from garpix_notify.tasks.tasks import send_system_notifications
    if created and instance.type == TYPE.SYSTEM:
        transaction.on_commit(lambda: send_system_notifications.delay(instance.pk))



from django.conf import settings

from garpix_notify.models.choices import CALL_URL, SMS_URL
from garpix_notify.models.config import NotifyConfig


class SendDataService:
    def __int__(self):
        try:
            config = NotifyConfig.get_solo()
            SMS_API_ID = config.sms_api_id
            SMS_LOGIN = config.sms_login
            SMS_PASSWORD = config.sms_password
            SMS_FROM = config.sms_from
        except Exception:
            SMS_API_ID = getattr(settings, 'SMS_API_ID', 1234567890)
            SMS_LOGIN = getattr(settings, 'SMS_LOGIN', '')
            SMS_PASSWORD = getattr(settings, 'SMS_PASSWORD', '')
            SMS_FROM = getattr(settings, 'SMS_FROM', '')

        try:
            config = NotifyConfig.get_solo()
            CALL_API_ID = config.call_api_id
            CALL_LOGIN = config.call_login
            CALL_PASSWORD = config.call_password
        except Exception:
            CALL_API_ID = getattr(settings, 'CALL_API_ID', 1234567890)
            CALL_LOGIN = getattr(settings, 'CALL_LOGIN', '')
            CALL_PASSWORD = getattr(settings, 'CALL_PASSWORD', '')

        self.url_dict_call = {
            0: '{url}?phone={to}&api_id={api_id}&json=1',
            1: '{url}?phone={to}&login={login}&password={password}&json=1',
            2: '{url}?login={login}&psw={password}&phones={to}&mes=code&call=1&fmt=3',
            3: '{url}?phone={to}&key={password}&service_id={login}',
        }

        self.operator_call = {
            0: {
                'url': CALL_URL.SMSRU_CALL_URL,
                'api_id': CALL_API_ID},
            1: {
                'url': CALL_URL.SMSRU_CALL_URL,
                'login': CALL_LOGIN,
                'password': CALL_PASSWORD},
            2: {
                'url': CALL_URL.SMSCENTRE_URL,
                'login': CALL_LOGIN,
                'password': CALL_PASSWORD},
            3: {
                'url': CALL_URL.UCALLER_URL,
                'login': CALL_LOGIN,
                'password': CALL_PASSWORD}}

        self.url_dict_sms = {
            0: '{url}?api_id={api_id}&to={to}&msg={text}&json=1',
            1: '{url}?login={user}&pwd={pwd}&phones={to}&message={text}&sender={from_text}',
            2: '{url}?login={user}&password={pwd}&phone={to}&text={text}',
            3: '{url}?login={user}&psw={pwd}&phones={to}&mes={text}',
            4: '{url}?login={user}&password={pwd}&txt={text}&to={to}',
            5: '{url}?method=push_msg&format=json&key={api_id}&text={text}&phone={to}&sender_name={from_text}',
            6: '{url}?user={user}&pwd={pwd}&sadr={from_text}&text={text}&dadr={to}',
        }

        self.operator_sms = {
            0: {
                'url': SMS_URL.SMSRU_URL,
                'api_id': SMS_API_ID},
            1: {
                'url': SMS_URL.INFOSMS_URL,
                'user': SMS_LOGIN,
                'pwd': SMS_PASSWORD,
                'from_text': SMS_FROM},
            2: {
                'url': SMS_URL.IQSMS_URL,
                'user': SMS_LOGIN,
                'pwd': SMS_PASSWORD},
            3: {
                'url': SMS_URL.SMSCENTRE_URL,
                'user': SMS_LOGIN,
                'pwd': SMS_PASSWORD},
            4: {
                'url': SMS_URL.SMS_SENDING_URL,
                'user': SMS_LOGIN,
                'pwd': SMS_PASSWORD},
            5: {
                'url': SMS_URL.SMS_PROSTO_URL,
                'user': SMS_LOGIN,
                'pwd': SMS_PASSWORD,
                'from_text': SMS_FROM},
            6: {
                'url': SMS_URL.WEBSZK_URL,
                'user': SMS_LOGIN,
                'pwd': SMS_PASSWORD,
                'from_text': SMS_FROM},
        }

    @staticmethod
    def response_check(response, operator_type, status):
        if status == 0:
            response_processing = {
                0 or 1: {
                    'Status': response.get('status'),
                    'Code': response.get('code'),
                    'Balance': response.get('balance'),
                    'ID_Call': response.get('call_id')},

                2: {'Status': response.get('id'),
                    'Code': response.get('code'),
                    'ID_Call': response.get('cnt'),
                    'Balance': response.get('balance')},

                3: {'Status': response.get('status'),
                    'Code': response.get('code'),
                    'Balance': response.get('balance'),
                    'ID_Call': response.get('unique_request_id')}}
            return response_processing[operator_type]

        if status == 1:
            response_processing = {
                0 or 1: {
                    'Status': response.get('status'),
                    'Status_code': response.get('status_code'),
                    'Status_text': response.get('status_text')},
                2: {'Status': response.get('status'),
                    'Status_code': response.get('error_code'),
                    'Status_text': response.get('error')},
                3: {'Status': response.get('status'),
                    'Status_code': response.get('code'),
                    'Status_text': response.get('error')}}
            return response_processing[operator_type]

        return None



import requests
from django.conf import settings
from django.utils.timezone import now

from garpix_notify.models.config import NotifyConfig
from garpix_notify.models.choices import STATE, SMS_URL
from garpix_notify.utils.receiving import ReceivingUsers
from garpix_notify.utils.send_data import SendDataService


class SMSClient:

    def __init__(self, notify):
        self.notify = notify
        try:
            self.config = NotifyConfig.get_solo()
            self.IS_SMS_ENABLED = self.config.is_sms_enabled
            self.SMS_URL_TYPE = self.config.sms_url_type
        except Exception:
            self.IS_SMS_ENABLED = getattr(settings, 'IS_SMS_ENABLED', True)
            self.SMS_URL_TYPE = getattr(settings, 'SMS_URL_TYPE', 0)

    def __client_sms(self):  # noqa

        if not self.IS_SMS_ENABLED:
            self.notify.state = STATE.DISABLED
            self.notify.to_log('Not sent (sending is prohibited by settings)')
            return

        try:
            send_data_service = SendDataService()
            users_list = self.notify.users_list.all()
            if not users_list.exists():
                phones: str = self.notify.phone
            else:
                phones_list: list = ReceivingUsers.run_receiving_users(users_list, 'phone')
                phones = ','.join(phones_list)
            msg = str(self.notify.text.replace(' ', '+'))
            url = send_data_service.url_dict_sms[self.SMS_URL_TYPE].format(
                **send_data_service.operator_sms[self.SMS_URL_TYPE], to=phones, text=msg)
            response = requests.get(url)
            response_dict = response.json()
            try:
                if self.SMS_URL_TYPE == SMS_URL.SMSRU_ID:
                    if response_dict['status'] == 'OK':
                        self.notify.to_log(
                            f"Статус основного запроса: {response_dict['status']}, Код статуса: \
                            {response_dict['status_code']}, Баланс: {response_dict['balance']}")
                        for key in response_dict['sms']:
                            if response_dict['sms'][key]['status'] == 'ERROR':
                                self.notify.to_log(
                                    f"Ошибка у абонента: Номер: {key}, Статус: {response_dict['sms'][key]['status']}, \
                                    Код статуса: {response_dict['sms'][key]['status_code']}, Описание: \
                                    {response_dict['sms'][key]['status_text']}")
                        self.notify.state = STATE.DELIVERED
                        self.notify.sent_at = now()
                    else:
                        self.notify.to_log(
                            f"Статус: {response_dict['status']}, Код статуса: {response_dict['status_code']}, "
                            f"Описание ошибки: {response_dict['status_text']}")
                        self.notify.state = STATE.REJECTED
                elif self.SMS_URL_TYPE == SMS_URL.WEBSZK_ID:
                    try:
                        int(response.text)
                        self.notify.state = STATE.DELIVERED
                        self.notify.sent_at = now()
                    except Exception as e:
                        self.notify.to_log(str(e))
                        self.notify.state = STATE.REJECTED
                elif self.SMS_URL_TYPE == SMS_URL.IQSMS_ID:
                    if response_dict['status'] == 'ok':
                        self.notify.to_log(
                            f"Статус: {response_dict['status']}, Код статуса: {response_dict['code']}, Описание: \
                            {response_dict['description']}")
                        self.notify.state = STATE.DELIVERED
                        self.notify.sent_at = now()
                    else:
                        self.notify.to_log(
                            f"Статус: {response_dict['status']}, Код статуса: {response_dict['code']}, "
                            f"Описание ошибки: {response_dict['description']}")
                        self.notify.state = STATE.REJECTED
                elif self.SMS_URL_TYPE == SMS_URL.INFOSMS_ID:
                    self.notify.state = STATE.DELIVERED
                    self.notify.sent_at = now()
                elif self.SMS_URL_TYPE == SMS_URL.SMSCENTRE_ID:
                    self.notify.state = STATE.DELIVERED
                    self.notify.sent_at = now()
                elif self.SMS_URL_TYPE == SMS_URL.SMS_SENDING_ID:
                    if response_dict['code'] == 1:
                        self.notify.to_log(
                            f"Статус: {response_dict['code']}, Описание: {response_dict['descr']}")
                        self.notify.state = STATE.DELIVERED
                        self.notify.sent_at = now()
                    else:
                        self.notify.to_log(
                            f"Статус: {response_dict['code']}, Описание ошибки: {response_dict['descr']}")
                        self.notify.state = STATE.REJECTED
                elif self.SMS_URL_TYPE == SMS_URL.SMS_PROSTO_ID:
                    if response_dict['response']['msg']['err_code'] == 0:
                        self.notify.to_log(
                            f"Статус: {response_dict['response']['msg']['err_code']}, Описание: \
                            {response_dict['response']['msg']['text']}")
                        self.notify.state = STATE.DELIVERED
                        self.notify.sent_at = now()
                    else:
                        self.notify.to_log(
                            f"Статус: {response_dict['response']['msg']['err_code']}, Описание ошибки: \
                            {response_dict['response']['msg']['text']}")
                        self.notify.state = STATE.REJECTED
            except Exception as e:
                self.notify.state = STATE.REJECTED
                self.notify.to_log(str(e))

        except Exception as e:  # noqa
            self.notify.state = STATE.REJECTED
            self.notify.to_log(str(e))

    @classmethod
    def send_sms(cls, notify):
        cls(notify).__client_sms()



import django
import os

from django.utils.crypto import get_random_string
from unittest import TestCase

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group

from garpix_notify.models import SystemNotify
from garpix_notify.models import NotifyCategory
from garpix_notify.models import NotifyTemplate
from garpix_notify.models import NotifyUserList
from garpix_notify.models import NotifyUserListParticipant
from garpix_notify.models.choices import TYPE


os.environ.setdefault("DJANGO_SETTINGS_MODULE", "app.settings")
django.setup()


class SystemNotifyTestCase(TestCase):

    def setUp(self):
        # test events
        self.PASS_TEST_EVENT = 0
        self.PASS_TEST_EVENT_1 = 1

        # data for test user
        self.data_user = {
            'username': 'email_test' + get_random_string(length=4),
            'email': 'test@garpix.com',
            'password': 'BlaBla123',
            'first_name': 'Ivan',
            'last_name': 'Ivanov',
        }

        # some data
        self.some_key = f"{get_random_string(length=4)}"
        self.some_text = f"{get_random_string(length=7)}"
        self.some_data_dict = {self.some_key: self.some_text}
        self.data_template_email = {
            'title': 'Тестовый системный темплейт',
            'subject': 'Тестовый темплейт',
            'text': self.some_data_dict,
            'html': self.some_data_dict,
            'type': TYPE.SYSTEM,
            'event': self.PASS_TEST_EVENT,
        }
        self.data_category = {
            'title': 'Тестовая категория_' + get_random_string(length=4),
            'template': '<div>{{text}}</div>',
        }
        super().setUp()

    def test_system_notify_positive(self):
        # Create user
        User = get_user_model()
        user = User.objects.create_user(**self.data_user)

        # Create category
        category = NotifyCategory.objects.create(**self.data_category)
        category = NotifyCategory.objects.filter(pk=category.pk).first()
        self.assertEqual(category.title, self.data_category['title'])
        self.assertEqual(category.template, self.data_category['template'])

        # Create template
        template_email = NotifyTemplate.objects.create(category=category, **self.data_template_email)
        self.assertEqual(template_email.title, self.data_template_email['title'])
        self.assertEqual(template_email.subject, self.data_template_email['subject'])
        self.assertEqual(template_email.text, self.data_template_email['text'])
        self.assertEqual(template_email.html, self.data_template_email['html'])
        self.assertEqual(template_email.type, self.data_template_email['type'])
        self.assertEqual(template_email.event, self.data_template_email['event'])
        self.assertEqual(template_email.category.id, category.id)

        # Create system notify
        SystemNotify.send(
            self.some_data_dict, user=user, event=self.PASS_TEST_EVENT
        )
        notify = SystemNotify.objects.filter(user=user, event=self.PASS_TEST_EVENT).first()
        data_compiled_email = {
            "title": f'{user} - room_{user.pk}',
            "text": f"{self.some_data_dict}, "
                    f'"event_id": {self.data_template_email["event"]}, '
                    f"user: {user.pk}",
            "type": TYPE.SYSTEM,
            "event": self.PASS_TEST_EVENT,
        }
        self.assertEqual(notify.event, template_email.event)
        self.assertEqual(notify.title, data_compiled_email["title"])
        # self.assertEqual(notify.data_json, data_compiled_email["text"])
        self.assertEqual(notify.type, data_compiled_email["type"])
        self.assertEqual(notify.event, data_compiled_email["event"])

    def test_system_notify_user_list(self):
        # test template data
        self.data_template_system_1 = {
            'title': 'Тестовый системный темплейт массовая рассылка',
            'subject': 'Тестовый темплейт',
            'text': f'{self.some_data_dict}',
            'html': f'{self.some_data_dict}',
            'type': TYPE.SYSTEM,
            'event': self.PASS_TEST_EVENT_1,
        }

        # Create user lists
        user_list = NotifyUserList.objects.create(title='userlist_' + get_random_string(length=4))
        group = Group.objects.create(name='group_' + get_random_string(length=4))
        user_list.user_groups.add(group)

        # Participant data
        self.user_list_participant_data = [
            {'user_list': user_list, 'email': 'test2@garpix.com'},
            {'user_list': user_list}
        ]
        for user_list_data in self.user_list_participant_data:
            NotifyUserListParticipant.objects.create(**user_list_data)

        # Create users
        user_model = get_user_model()
        user_data_list = [
            {
                'username': f'user_{user_count}_' + get_random_string(length=3),
                'email': f'{get_random_string(length=5).capitalize()}@garpix.com',
                'password': f'1{get_random_string(length=9)}1',
                'first_name': get_random_string(length=5),
                'last_name': get_random_string(length=5),
            } for user_count in range(10)
        ]
        for user_data in user_data_list:
            user = user_model.objects.create_user(**user_data)
            user.groups.add(group)
            user.save()

        # Create category
        category = NotifyCategory.objects.create(**self.data_category)
        self.assertEqual(category.title, self.data_category['title'])
        self.assertEqual(category.template, self.data_category['template'])

        # Create template
        template_notify = NotifyTemplate.objects.create(category=category, **self.data_template_system_1)
        template_notify = NotifyTemplate.objects.get(pk=template_notify.pk)
        template_notify.user_lists.add(user_list)
        template_notify.save()

        self.assertEqual(template_notify.title, self.data_template_system_1['title'])
        self.assertEqual(template_notify.subject, self.data_template_system_1['subject'])
        self.assertEqual(template_notify.text, self.data_template_system_1['text'])
        self.assertEqual(template_notify.html, self.data_template_system_1['html'])
        self.assertEqual(template_notify.type, self.data_template_system_1['type'])
        self.assertEqual(template_notify.event, self.data_template_system_1['event'])
        self.assertEqual(template_notify.category.id, category.id)

        # Create system notify
        SystemNotify.send(self.some_data_dict, event=self.PASS_TEST_EVENT_1)
        notify_qs = SystemNotify.objects.select_related('user').filter(
            event=self.PASS_TEST_EVENT_1
        ).order_by('user__id')

        # Test system notify
        for notify in notify_qs:
            user = user_model.objects.filter(pk=notify.user.pk).first()
            self.data_compiled_system_1 = {
                'title': f'{user.username} - room_{user.pk}',
                'text': f'{self.some_data_dict}, '
                        f'event_id: {self.data_template_email["event"]}, '
                        f'user: {user.pk}',
                'type': TYPE.SYSTEM,
                'event': self.PASS_TEST_EVENT_1,
            }

            # self.assertEqual(notify.title, self.data_compiled_system_1['title'])
            # self.assertEqual(notify.data_json, self.data_compiled_system_1['text'])
            self.assertEqual(notify.type, self.data_compiled_system_1['type'])
            self.assertEqual(notify.event, self.data_compiled_system_1['event'])



import django
import os

from django.utils.crypto import get_random_string
from unittest import TestCase

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group

from garpix_notify.models import Notify
from garpix_notify.models import NotifyCategory
from garpix_notify.models import NotifyTemplate
from garpix_notify.models import NotifyUserList
from garpix_notify.models import NotifyUserListParticipant
from garpix_notify.models.choices import TYPE

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "app.settings")
django.setup()


class PreBuildTestCase(TestCase):
    def setUp(self):
        # нулевой евент, только для теста
        self.PASS_TEST_EVENT = 0
        self.PASS_TEST_EVENT_1 = 1
        self.PASS_TEST_EVENT_2 = 2
        # тестовый пользователь
        self.data_user = {
            'username': 'email_test' + get_random_string(length=4),
            'email': 'test@garpix.com',
            'password': 'BlaBla123',
            'first_name': 'Ivan',
            'last_name': 'Ivanov',
        }
        # data
        self.data_template_email = {
            'title': 'Тестовый темплейт',
            'subject': 'Тестовый темплейт {{user.email}}',
            'text': 'Контент текстовый {{user.email}} - {{sometext}}',
            'html': 'Контент HTML {{user.email}} - {{sometext}}',
            'type': TYPE.EMAIL,
            'event': self.PASS_TEST_EVENT,
        }
        self.data_category = {
            'title': 'Основная категория_' + get_random_string(length=4),
            'template': '<div>{{text}}</div>',
        }
        self.sometext = 'bla bla'
        self.data_compiled_email = {
            'subject': f'Тестовый темплейт {self.data_user["email"]}',
            'text': f'Контент текстовый {self.data_user["email"]} - {self.sometext}',
            'html': f'Контент HTML {self.data_user["email"]} - {self.sometext}',
            'type': TYPE.EMAIL,
            'event': self.PASS_TEST_EVENT,
        }
        super().setUp()

    def test_notify_email_positive(self):
        # Создание пользователя
        User = get_user_model()
        user = User.objects.create_user(**self.data_user)
        # Создание категории
        category = NotifyCategory.objects.create(**self.data_category)
        category = NotifyCategory.objects.get(pk=category.pk)
        self.assertEqual(category.title, self.data_category['title'])
        self.assertEqual(category.template, self.data_category['template'])
        # Создание шаблона
        template_email = NotifyTemplate.objects.create(category=category, **self.data_template_email)
        template_email = NotifyTemplate.objects.get(pk=template_email.pk)
        self.assertEqual(template_email.title, self.data_template_email['title'])
        self.assertEqual(template_email.subject, self.data_template_email['subject'])
        self.assertEqual(template_email.text, self.data_template_email['text'])
        self.assertEqual(template_email.html, self.data_template_email['html'])
        self.assertEqual(template_email.type, self.data_template_email['type'])
        self.assertEqual(template_email.event, self.data_template_email['event'])
        self.assertEqual(template_email.category.id, category.id)
        # Создание пробного письма
        notify = Notify.send(self.PASS_TEST_EVENT, {
            'sometext': self.sometext,
            'user': user,
        }, user=user)
        self.assertEqual(notify[0].event, template_email.event)
        self.assertEqual(notify[0].subject, self.data_compiled_email['subject'])
        self.assertEqual(notify[0].text, self.data_compiled_email['text'])
        self.assertEqual(notify[0].html, self.data_compiled_email['html'])
        self.assertEqual(notify[0].type, self.data_compiled_email['type'])
        self.assertEqual(notify[0].event, self.data_compiled_email['event'])

    def test_notify_email_user_list(self):
        self.data_template_email_1 = {
            'title': 'Тестовый темплейт',
            'subject': 'Тестовый темплейт {{user.email}}',
            'text': 'Контент текстовый {{user.email}} - {{sometext}}',
            'html': 'Контент HTML {{user.email}} - {{sometext}}',
            'type': TYPE.EMAIL,
            'event': self.PASS_TEST_EVENT_1,
        }
        self.data_compiled_email_1 = {
            'subject': f'Тестовый темплейт {self.data_user["email"]}',
            'text': f'Контент текстовый {self.data_user["email"]} - {self.sometext}',
            'html': f'Контент HTML {self.data_user["email"]} - {self.sometext}',
            'type': TYPE.EMAIL,
            'event': self.PASS_TEST_EVENT_1
        }
        # Создание пользователя
        User = get_user_model()
        user = User.objects.create_user(**self.data_user)
        # Создание списка
        user_list = NotifyUserList.objects.create(title='userlist_' + get_random_string(length=4))
        group = Group.objects.create(name='group_' + get_random_string(length=4))
        user_list.user_groups.add(group)

        user_list_participant1 = NotifyUserListParticipant.objects.create(  # noqa
            user_list=user_list,
            email='test2@garpix.com',
        )
        user_list_participant2 = NotifyUserListParticipant.objects.create(  # noqa
            user_list=user_list,
        )
        user_list_participant3 = NotifyUserListParticipant.objects.create(  # noqa
            user_list=user_list,
            email='test3@garpix.com',
        )

        # Создание категории
        category = NotifyCategory.objects.create(**self.data_category)
        category = NotifyCategory.objects.get(pk=category.pk)
        self.assertEqual(category.title, self.data_category['title'])
        self.assertEqual(category.template, self.data_category['template'])

        # Создание шаблона
        template_email = NotifyTemplate.objects.create(category=category, **self.data_template_email_1)
        template_email = NotifyTemplate.objects.get(pk=template_email.pk)
        template_email.user_lists.add(user_list)
        template_email.save()

        self.assertEqual(template_email.title, self.data_template_email_1['title'])
        self.assertEqual(template_email.subject, self.data_template_email_1['subject'])
        self.assertEqual(template_email.text, self.data_template_email_1['text'])
        self.assertEqual(template_email.html, self.data_template_email_1['html'])
        self.assertEqual(template_email.type, self.data_template_email_1['type'])
        self.assertEqual(template_email.event, self.data_template_email_1['event'])
        self.assertEqual(template_email.category.id, category.id)

        # Создание пробного письма
        notify = Notify.send(self.PASS_TEST_EVENT_1, {
            'sometext': self.sometext,
            'user': user,
        }, user=user)

        self.assertEqual(notify[0].event, template_email.event)

        # notify
        self.assertEqual(notify[0].subject, self.data_compiled_email_1['subject'])
        self.assertEqual(notify[0].text, self.data_compiled_email_1['text'])
        self.assertEqual(notify[0].html, self.data_compiled_email_1['html'])
        self.assertEqual(notify[0].type, self.data_compiled_email_1['type'])
        self.assertEqual(notify[0].event, self.data_compiled_email_1['event'])
        self.assertEqual(notify[0].email, 'test@garpix.com')

    def test_notify_viber(self):
        self.data_template_viber = {
            'title': 'Тестовый темплейт',
            'subject': 'Тестовый темплейт {{user.viber_chat_id}}',
            'text': 'Контент текстовый {{user.viber_chat_id}} - {{sometext}}',
            'html': 'Контент HTML {{user.viber_chat_id}} - {{sometext}}',
            'type': TYPE.VIBER,
            'event': self.PASS_TEST_EVENT_2,
        }
        self.data_viber_user = {
            'username': 'viber_' + get_random_string(length=5),
            'viber_secret_key': '111',
            'viber_chat_id': 'm4FsaRu5kBi8HzSAC0liFQ==',
            'password': 'BlaBla123',
            'first_name': 'IvanViber',
            'last_name': 'IvanovViber',
        }
        self.data_compiled_viber = {
            'subject': f'Тестовый темплейт {self.data_viber_user["viber_chat_id"]}',
            'text': f'Контент текстовый {self.data_viber_user["viber_chat_id"]} - {self.sometext}',
            'html': f'Контент HTML {self.data_viber_user["viber_chat_id"]} - {self.sometext}',
            'type': TYPE.VIBER,
            'event': self.PASS_TEST_EVENT_2,
        }
        # Создание пользователя
        User = get_user_model()
        user = User.objects.create_user(**self.data_viber_user)
        # Создание списка
        user_list = NotifyUserList.objects.create(title='viber_' + get_random_string(length=4))
        user_list_participant1 = NotifyUserListParticipant.objects.create(  # noqa
            user_list=user_list,
        )
        # Создание категории
        category = NotifyCategory.objects.create(**self.data_category)
        category = NotifyCategory.objects.get(pk=category.pk)

        self.assertEqual(category.title, self.data_category['title'])
        self.assertEqual(category.template, self.data_category['template'])

        # Создание шаблона
        template_viber = NotifyTemplate.objects.create(**self.data_template_viber, category=category)
        template_viber = NotifyTemplate.objects.get(pk=template_viber.pk)
        template_viber.user_lists.add(user_list)
        template_viber.save()

        self.assertEqual(template_viber.title, self.data_template_viber['title'])
        self.assertEqual(template_viber.subject, self.data_template_viber['subject'])
        self.assertEqual(template_viber.text, self.data_template_viber['text'])
        self.assertEqual(template_viber.html, self.data_template_viber['html'])
        self.assertEqual(template_viber.type, self.data_template_viber['type'])
        self.assertEqual(template_viber.event, self.data_template_viber['event'])
        self.assertEqual(template_viber.category.id, category.id)

        # Создание уведомления
        notify = Notify.send(self.PASS_TEST_EVENT_2, {
            'sometext': self.sometext,
            'user': user,
        }, user=user)

        self.assertEqual(notify[0].event, template_viber.event)

        # notify
        self.assertEqual(notify[0].subject, self.data_compiled_viber['subject'])
        self.assertEqual(notify[0].text, self.data_compiled_viber['text'])
        self.assertEqual(notify[0].html, self.data_compiled_viber['html'])
        self.assertEqual(notify[0].type, self.data_compiled_viber['type'])
        self.assertEqual(notify[0].event, self.data_compiled_viber['event'])
        self.assertEqual(notify[0].user.viber_chat_id, 'm4FsaRu5kBi8HzSAC0liFQ==')



import requests
import enum
from django.conf import settings
from django.utils.timezone import now

from garpix_notify.models.config import NotifyConfig
from garpix_notify.models.choices import STATE, CALL_URL
from garpix_notify.utils.send_data import SendDataService


class CallClient:

    class ChoiceValue(enum.Enum):
        OK = 0
        BAD = 1

    def __init__(self, notify):
        self.notify = notify
        try:
            self.config = NotifyConfig.get_solo()
            self.IS_CALL_ENABLED = self.config.is_call_enabled
            self.CALL_URL_TYPE = self.config.call_url_type
        except Exception:
            self.IS_CALL_ENABLED = getattr(settings, 'IS_CALL_ENABLED', True)
            self.CALL_URL_TYPE = getattr(settings, 'CALL_URL_TYPE', 0)

    def _value_checker(self, response_dict):
        value = None
        if self.CALL_URL_TYPE in [CALL_URL.SMSRU_CALL_ID, CALL_URL.SMSRU_CALL_API_ID]:
            if response_dict['status'] == "OK":
                value = self.ChoiceValue.OK.value
            else:
                value = self.ChoiceValue.BAD.value

        elif self.CALL_URL_TYPE == CALL_URL.SMSCENTRE_ID:
            if response_dict['error']:
                value = self.ChoiceValue.BAD.value
            else:
                value = self.ChoiceValue.OK.value

        elif self.CALL_URL_TYPE == CALL_URL.UCALLER_ID:
            if response_dict['status']:
                value = self.ChoiceValue.OK.value
            else:
                value = self.ChoiceValue.BAD.value
        return value

    def __send_call_code(self):  # noqa
        if not self.IS_CALL_ENABLED:
            self.notify.state = STATE.DISABLED
            self.notify.to_log('Not sent (sending is prohibited by settings)')
            return

        phone = f'{self.notify.phone}'

        send_data_service = SendDataService()

        try:
            url = send_data_service.url_dict_call[self.CALL_URL_TYPE].format(**send_data_service.operator_call[self.CALL_URL_TYPE], to=phone)
            response_url = requests.get(url)
            response_dict = response_url.json()
            value = self._value_checker(response_dict)

            response = send_data_service.response_check(response=response_dict, operator_type=self.CALL_URL_TYPE, status=value)

            self.__save_to_log(response=response, value=value)
        except Exception as e:
            self.notify.state = STATE.REJECTED
            self.notify.to_log(str(e))

    def __save_to_log(self, response, value):
        if value == self.ChoiceValue.OK.value:
            self.notify.to_log(
                'Status: {Status}, Code: {Code}, Balance: {Balance}, ID_Call: {ID_Call}'.format(**response))
            self.notify.state = STATE.DELIVERED
            self.notify.sent_at = now()
        elif value == self.ChoiceValue.BAD.value:
            self.notify.to_log(
                'Status: {Status}, Status_code: {Status_code}, Status_text: {Status_text}'.format(**response))
            self.notify.state = STATE.REJECTED
        else:
            return None

    @classmethod
    def get_value_checker(cls, response_dict):
        return cls(notify=None)._value_checker(response_dict)

    @classmethod
    def get_url_type(cls):
        return cls(notify=None).CALL_URL_TYPE

    @classmethod
    def send_call(cls, notify):
        cls(notify).__send_call_code()



from django.utils.html import format_html

from garpix_notify.models.system_log import SystemNotifyErrorLog
from garpix_notify.models.log import NotifyErrorLog
from garpix_notify.models.choices import STATE


class NotifyMethodsMixin:

    def get_format_state(self):
        if self.state == STATE.WAIT:
            return format_html('<span style="color:orange;">В ожидании</span>')
        elif self.state == STATE.DELIVERED:
            return format_html('<span style="color:green;">Отправлено</span>')
        elif self.state == STATE.REJECTED:
            return format_html('<span style="color:red;">Отклонено</span>')
        elif self.state == STATE.DISABLED:
            return format_html('<span style="color:red;">Отправка запрещена</span>')
        return format_html('<span style="color:black;">Неизвестный статус</span>')

    get_format_state.short_description = 'Статус'

    def to_log(self, error_text):
        from garpix_notify.models.notify import Notify
        if isinstance(self, Notify):
            log = NotifyErrorLog(notify=self, error=error_text)
        else:
            log = SystemNotifyErrorLog(notify=self, error=error_text)
        log.save()



from smtplib import SMTP, SMTP_SSL
from typing import Optional

from email.header import Header
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.application import MIMEApplication

from django.conf import settings
from django.utils.timezone import now
from django.utils.safestring import mark_safe
from django.template import Template, Context

from garpix_notify.models.config import NotifyConfig
from garpix_notify.models.smtp import SMTPAccount
from garpix_notify.models.choices import STATE, EMAIL_MALLING
from garpix_notify.utils import ReceivingUsers


class EmailClient:

    def __init__(self, notify):
        self.notify = notify
        try:
            self.config = NotifyConfig.get_solo()
            self.IS_EMAIL_ENABLED = self.config.is_email_enabled
            self.EMAIL_MALLING_TYPE = self.config.email_malling
        except Exception:
            self.IS_EMAIL_ENABLED = getattr(settings, 'IS_EMAIL_ENABLED', True)
            self.EMAIL_MALLING_TYPE = getattr(settings, 'EMAIL_MALLING', EMAIL_MALLING.BCC)

    def __get_valid_smtp_account(self) -> Optional[SMTPAccount]:
        account = SMTPAccount.get_free_smtp()
        if account is None or not account.is_active:
            return

        if self.notify.sender_email is None:
            self.notify.sender_email = account.sender

        return account

    def __render_body(self, mail_from, layout, emails):
        msg = MIMEMultipart('mixed')
        if len(emails) > 1:
            if self.EMAIL_MALLING_TYPE == EMAIL_MALLING.BCC:
                msg['BCC'] = ', '.join(emails)
            else:
                msg['СС'] = ', '.join(emails)
        else:
            msg['To'] = ''.join(emails)
        msg['Subject'] = Header(self.notify.subject, 'utf-8')
        msg['From'] = mail_from

        text = MIMEText(self.notify.text, 'plain', 'utf-8')
        msg.attach(text)

        if self.notify.html:
            template = Template(layout.template)
            context = Context({'text': mark_safe(self.notify.html)})
            html = MIMEText(mark_safe(template.render(context)), 'html', 'utf-8')
            msg.attach(html)

        for fl in self.notify.files.all():
            with fl.file.open(mode='rb') as f:
                part = MIMEApplication(
                    f.read(),
                    Name=fl.file.name.split('/')[-1]
                )

            part['Content-Disposition'] = 'attachment; filename="%s"' % fl.file.name.split('/')[-1]

            msg.attach(part)

        return msg

    def __send_email_client(self):
        if not self.IS_EMAIL_ENABLED:
            self.notify.state = STATE.DISABLED
            self.notify.to_log('Not sent (sending is prohibited by settings)')
            return

        emails = []
        account = self.__get_valid_smtp_account()

        if account is None:
            self.notify.state = STATE.DISABLED
            self.notify.to_log('No SMTPAccount')
            return

        users_list = self.notify.users_list.all()

        if users_list.exists():
            emails: list = ReceivingUsers.run_receiving_users(users_list, 'email')

        if self.notify.email:
            emails.append(self.notify.email)

        try:
            body = self.__render_body(mail_from=account.sender, layout=self.notify.category, emails=emails)
            server = SMTP_SSL(account.host, account.port) if account.is_use_ssl else SMTP(account.host, account.port)
            server.ehlo()
            if account.is_use_tls:
                server.starttls()
            server.login(account.username, account.password)
            server.sendmail(account.sender, emails, body.as_string())
            server.close()
            self.notify.state = STATE.DELIVERED
            self.notify.sent_at = now()
        except Exception as e:
            self.notify.state = STATE.REJECTED
            self.notify.to_log(str(e))

    @classmethod
    def send_email(cls, notify):
        cls(notify).__send_email_client()



from django.contrib.auth import get_user_model


class ReceivingUsers:

    def __init__(self, users_list: list, value: str = None):
        self.users_list = users_list
        self.value = value

    def __returning_specific_list(self, receivers: list) -> list:
        # Убираем дубликаты пользователей для переданного значения из функции
        receivers_dict = list({v[self.value]: v for v in receivers}.values())
        receivers_new = [user.get(self.value) for user in receivers_dict]
        return receivers_new

    def __receiving_users(self) -> list:  # noqa: C901
        user_model = get_user_model()
        receivers: list = []
        for user_list in self.users_list:
            # Проверяем рассылку, не отмечена ли массовая рассылка
            if user_list.mail_to_all:
                users = user_model.objects.all()
                receivers.extend([
                    {'user': user,
                     'email': user.email,
                     'phone': user.phone,
                     'viber_chat_id': user.viber_chat_id
                     } for user in users
                ])

                if self.value:
                    return self.__returning_specific_list(receivers)
                return receivers

            # Собираем данные из дополнительного списка получателей, если он есть.
            users_participants = user_list.participants.all()
            if users_participants.exists():
                for participant in users_participants:
                    if participant.user or participant.email or participant.phone or participant.viber_chat_id:
                        receivers.append({
                            'user': participant.user,
                            'email': participant.user.email if participant.user else participant.email,
                            'phone': participant.user.phone if participant.user else participant.phone,
                            'viber_chat_id':
                                participant.user.viber_chat_id if participant.user else participant.viber_chat_id
                        })

            # Собираем данные из групп, которые входят в список рассылки
            user_groups_qs = user_list.user_groups.all()
            if user_groups_qs.exists():
                group_users = user_model.objects.prefetch_related('groups').filter(groups__in=user_groups_qs)

                if group_users.exists():
                    receivers.extend([
                        {'user': user,
                         'email': user.email,
                         'phone': user.phone,
                         'viber_chat_id': user.viber_chat_id
                         } for user in group_users
                    ])

            # Собираем данные по пользователям, если они были переданы
            users_qs = user_list.users.all()
            if users_qs.exists():
                receivers.extend([
                    {
                        'user': user,
                        'email': user.email,
                        'phone': user.phone,
                        'viber_chat_id': user.viber_chat_id
                    } for user in users_qs
                ])

        if self.value:
            return self.__returning_specific_list(receivers)

        return receivers

    @classmethod
    def run_receiving_users(cls, users_list: list, value: str = None) -> list:
        return cls(users_list, value).__receiving_users()
